<?php

/**
 * @author Jan Hruby
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package moodle multiauth
 *
 * Authentication Plugin: KeyShield SSO Authentication
 *
 * Authentication using KeyShield SSO.
 *
 * 2012-10-24  File created.
 */

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.');    ///  It must be included from a Moodle page
}

require_once($CFG->libdir.'/authlib.php');


class KShieldCredentials
{
  function KShieldCredentials($username)
  {
    $this->username = $username; 
    $this->password = "";
  }
}
/**
 * KeyShield SSO authentication plugin.
 */
class auth_plugin_kshield extends auth_plugin_base {
     function init_plugin($authtype) {
        $this->pluginconfig = 'auth/'.$authtype;
        $this->config = get_config($this->pluginconfig);
     }
    /**
     * Constructor.
     */
    function auth_plugin_kshield() {
        $this->authtype = 'kshield';
        $this->roleauth = 'auth_kshield';
        $this->errorlogtag = '[AUTH KSHIELD] ';
        $this->init_plugin($this->authtype);
    }

    function prevent_local_passwords() {
        return true;
    }

    /**
     * Authenticates user against WSTrust
     * Returns true if the username and password work and false if they are
     * wrong or don't exist.
     *
     * @param string $username The username (with system magic quotes)
     * @param string $password The password (with system magic quotes)
     * @return bool Authentication success or failure.
     */
    function user_login ($username, $password) {
        return false;
    }

    /**
     * Returns true if this authentication plugin is 'internal'.
     *
     * @return bool
     */
    function is_internal() {
        return false;
    }

    /**
     * Returns true if this authentication plugin can change the user's
     * password.
     *
     * @return bool
     */
    function can_change_password() {
        return false;
    }

    /**
     * Authentication choice (KeyShield SSO or other)
     * Redirection to the KeyShield SSO  form or to login/index.php
     * for other authentication
     */
    function loginpage_hook() {
      global $CFG;
      
      if ($_SERVER['REQUEST_METHOD'] === 'GET')
      {
        $ip = $this->getClientIP();
        $username =  $this->getUsernameByIP($this->config->kshieldURL, $ip, $this->config->kshieldApiKey, $this->config->kshieldUserAttr);
        
        if (!empty($username))
        {
          $GLOBALS["frm"] = new KShieldCredentials($username);
          $user =  get_complete_user_data('username', $username, $CFG->mnet_localhost_id);
          
          if (!$user)
          {
             if (empty($CFG->authpreventaccountcreation)) {
                $user = create_user_record($username, "", "ldap");
            } 
          }
          else
          {
            if (!empty($user->suspended)) {
              add_to_log(SITEID, 'login', 'error', 'index.php', $username);
              error_log('[client '.getremoteaddr()."]  $CFG->wwwroot  Suspended Login:  $username  ".$_SERVER['HTTP_USER_AGENT']);
              $user = false;
            }
          }
          
          
          $GLOBALS["user"] = $user;
        }
      }
    }

    /**
     * Prints a form for configuring this authentication plugin.
     *
     * This function is called from admin/auth.php, and outputs a full page with
     * a form for configuring this plugin.
     *
     * @param array $page An object containing all the data for this page.
     */
    function config_form($config, $err, $user_fields) {
        global $CFG, $OUTPUT;

        if (!function_exists('curl_init')) { // Is curl really there?
          echo $OUTPUT->notification(get_string('curl_noextension', 'auth_kshield'));
        }
        
        if (!function_exists('json_decode'))
        {
          echo $OUTPUT->notification(get_string('json_decode_noextension', 'auth_kshield'));
        }

        include($CFG->dirroot.'/auth/kshield/config.html');
    }

    
    /**
     * A chance to validate form data, and last chance to
     * do stuff before it is inserted in config_plugin
     * @param object object with submitted configuration settings (without system magic quotes)
     * @param array $err array of error messages
     */
    function validate_form($form, &$err) {
      $json_url = $form->kshieldURL.'/json/userByIP/test?key='.$form->kshieldApiKey;
      $ch = curl_init( $json_url );
    
      $options = array(
        CURLOPT_RETURNTRANSFER => true
        );
    
      curl_setopt_array( $ch, $options );
    
      $result =  curl_exec($ch);
      $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    
      if ($result) {
        $obj = json_decode($result,true); 
        if (!array_key_exists('ipAddress', $obj)) {
          $err['kshieldURL'] = get_string('err_invalid_server_response', 'auth_kshield');
        }
        elseif ($httpCode != 200) {
          $err['kshieldApiKey'] = get_string('err_invalid_api_key', 'auth_kshield');
        }
      }
      else
      {
        $err['kshieldURL'] = get_string('err_unable_to_connect', 'auth_kshield');
      }
	  
	  $clientIpAttr = $form->kshieldClientIpSrvAttr;
	  if ($clientIpAttr && (
	      $clientIpAttr === "REMOTE_ADDR" && !isset($_SERVER["REMOTE_ADDR"]) ||
		  $clientIpAttr === "HTTP_X_FORWARDED_FOR" && !isset($_SERVER["HTTP_X_FORWARDED_FOR"]) ||
		  $clientIpAttr === "HTTP_X_REAL_IP" && !isset($_SERVER["HTTP_X_REAL_IP"]) ||
		  $clientIpAttr === "HTTP_CLIENT_IP" && !isset($_SERVER["HTTP_CLIENT_IP"])
		  )) {	  
		$err['kshieldClientIpSrvAttr'] = get_string('err_invalid_ip_srv_prop', 'auth_kshield');
	  }
    }
    /**
     * Returns the URL for changing the user's pw, or empty if the default can
     * be used.
     *
     * @return moodle_url
     */
    function change_password_url() {
        return null;
    }

    /**
     * Processes and stores configuration data for this authentication plugin.
     */
    function process_config($config) {
        // KeyShield SSO settings
        if (!isset($config->kshieldURL)) {
            $config->kshieldURL = '';
        }
        if (!isset($config->kshieldApiKey)) {
        	$config->kshieldApiKey = '';
        }
        if (!isset($config->kshieldUserAttr)) {
        	$config->kshieldUserAttr = '';
        }
		if (!isset($config->kshieldClientIpSrvAttr)) {
        	$config->kshieldClientIpSrvAttr = '';
        }
               
        // save KeyShield SSO settings
        set_config('kshieldURL', trim($config->kshieldURL), $this->pluginconfig);
        set_config('kshieldApiKey', trim($config->kshieldApiKey), $this->pluginconfig);
        set_config('kshieldUserAttr', trim($config->kshieldUserAttr), $this->pluginconfig);
		set_config('kshieldClientIpSrvAttr', $config->kshieldClientIpSrvAttr, $this->pluginconfig);
        return true;
    }


    function iscreator($username) {
        return false;
    }


    function get_userinfo($username) {
        if (empty($this->config->host_url)) {
            return array();
        }
        return parent::get_userinfo($username);
    }


    function sync_users($do_updates=true) {
    }
  
  function getClientIP() {
	$clientIpAttr = $this->config->kshieldClientIpSrvAttr;
	  if ($clientIpAttr) {
        switch (clientIpAttr) {
          case "REMOTE_ADDR":
		    if (isset($_SERVER["REMOTE_ADDR"])) {
              return $_SERVER["REMOTE_ADDR"] ;
			}
			else {
			  return "";
			}
            break;

          case "HTTP_X_FORWARDED_FOR":
		    if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])) {
              return $_SERVER["HTTP_X_FORWARDED_FOR"] ;
			}
			else {
			  return "";
			}
            break;

          case "HTTP_X_REAL_IP":
		    if (isset($_SERVER["HTTP_X_REAL_IP"])) {
              return $_SERVER["HTTP_X_REAL_IP"] ;
			}
			else {
			  return "";
			}
            break;

          case "HTTP_CLIENT_IP":
		    if (isset($_SERVER["HTTP_CLIENT_IP"])) {
              return $_SERVER["HTTP_CLIENT_IP"] ;
			}
			else {
			  return "";
			}
            break;
        }
	  }
	  
    if ( isset($_SERVER["REMOTE_ADDR"]) ) {
      return $_SERVER["REMOTE_ADDR"] ;
    } else if ( isset($_SERVER["HTTP_X_FORWARDED_FOR"]) ) {
      return $_SERVER["HTTP_X_FORWARDED_FOR"] ;
    } else if ( isset($_SERVER["HTTP_X_REAL_IP"]) ) {
      return $_SERVER["HTTP_X_REAL_IP"] ;
	} else if ( isset($_SERVER["HTTP_CLIENT_IP"]) ) {
      return $_SERVER["HTTP_CLIENT_IP"] ;
    }
	
    return null;
  }

  function appendURLParam($params, $paramName, $paramValue)
  {
		if (!isset($paramName) || $paramName == '' || !isset($paramValue) || $paramValue == '') {
			return $params;
		}
  
  	if (isset($params) && $params!='') {
  		$params = $params.'&';
  	}
  	else {
  		$params = '?';
  	}	
		return $params.$paramName.'='.$paramValue;
  }
  
	function getUsernameByIP($ssoUrl, $ip, $apiKey, $userAttr)
	{
		$ssoParams = $this->appendURLParam($ssoParams, "key", $apiKey);
		$ssoParams = $this->appendURLParam($ssoParams, "attributes", $userAttr);
		
		$json_url = $ssoUrl.'/json/userByIP/'.$ip.$ssoParams;
		
		$options = array(
			CURLOPT_RETURNTRANSFER => true
			);
		
		$ch = curl_init( $json_url );
		curl_setopt_array( $ch, $options );
		
		$result =  curl_exec($ch);
		
		if ($result) {
			$obj = json_decode($result,true);
			
			if(!is_null($userAttr) && $userAttr!='') {
				if ($obj["attributes"]) {
					$attrs = $obj["attributes"];
					if ($attrs[$userAttr]) {
						return $attrs[$userAttr];
					}
				}
			}
			else {
				if ($obj["screenName"]) {
					return $obj["screenName"];
				}
			}
		}
		return null;
	}
}
